#include <stdio.h>
#include <stdint.h>
#include <math.h>

#ifdef _WIN32
    #include <time.h>
    void timeBySeconds(double *currTime, double prevTime)
    {
        *currTime = (double)clock() / CLOCKS_PER_SEC - prevTime;
    }
#else
    #ifndef __USE_POSIX199309
        #define __USE_POSIX199309
    #endif
    #include <time.h>
    void timeBySeconds(double *currTime, double prevTime)
    {
        struct timespec ts;
        clock_gettime(CLOCK_MONOTONIC, &ts);
        *currTime = (double)ts.tv_sec + ts.tv_nsec*1e-9 - prevTime;
    }
#endif


int main(int argc, char* argv[])
{
#ifndef M_PI
    const double M_PI = 3.141592653589793238;
#endif
    const double dx_ub = 2. * M_PI;
    const double dx_lb = 0.;

    double start, finish;

    printf("+- Interior Points -+-- Computed Value -+\n");
    timeBySeconds(&start, 0.);
    
    for(int j=2; j<=29; j++)
    {
        const int N = 1 << j;
        const double step = (dx_ub - dx_lb) / N;
        
        double sum = 0.;

        sum += fabs(sin(dx_lb)) * step / 2.;
        for(int i=1; i<=N-1; i++)
        {
            const double dx = i * step;
            sum += fabs(sin(dx)) * step;
        }
        sum += fabs(sin(dx_ub)) * step / 2.;
        
        printf("|     %10d    |   %14.6e  |\n%s", N, sum,
            "+-------------------+-------------------+\n");
    }

    timeBySeconds(&finish, start);
    printf("\n    Elapsed time: %8.3f [seconds]\n", finish);

    return 0;
}
